precision highp float;

uniform virtualMipMap2D u_image;// use textureVirtualMipMap2D to access it, it's magic. See GlProgramProcessor for more details.
uniform #ABSOLUTE float u_blurRadius;
uniform #RELATIVE float u_gradientSize;
uniform #RELATIVE float u_size;

uniform #RELATIVE vec2 u_startPosition;
uniform vec2 u_delta;
uniform vec2 u_texSize;

varying vec2 v_texCoord;

const float blurSteps = 5.0;
const float EPSILON = 0.000001;

void main() {
    vec4 color = vec4(0.0);
    float total = 0.0;
    float aspect = u_texSize.x / u_texSize.y;

    vec2 diff = v_texCoord - u_startPosition;
    diff.y /= aspect;

    // Acually the same as distance(), but distance() is broken on some divices, with no highp prescison.
    float pointDistance = sqrt(diff.x * diff.x + diff.y * diff.y);

    float gradientSize = u_gradientSize / aspect;
    float size = u_size / aspect;
    float radius = u_blurRadius * smoothstep(
        0.0,
        1.0,
        (abs(pointDistance) - size) / gradientSize
    );

    float percent;
    float weight;
    vec4 samplePix;

    float lod = max(log2(radius / blurSteps), 0.0);
    for (float t = -blurSteps; t <= blurSteps; t++) {
        percent = t / blurSteps;

        samplePix = textureVirtualMipMap2D(u_image, v_texCoord + u_delta * percent * radius / u_texSize, lod);

        weight = (1.0 - abs(percent));
        color += samplePix * weight;
        total += weight;
    }

    color /= max(total, EPSILON);

    gl_FragColor = clamp(color, 0.0, 1.0);
}